import 'dart:developer';

import 'package:flutter/services.dart';
import 'package:hypersnapsdk_flutter/hyper_snap_sdk_constants.dart';

import 'hv_result.dart';

/// The class that provides methods for barcode capture using HyperSnap SDK.
class HVBarcodeScanCapture {
  static const MethodChannel _hvBarcodeScanCaptureChannel =
      MethodChannel(HyperSnapSDKConstants.hyperSnapChannel);

  /// Starts the barcode scanning and capture process.
  ///
  /// Use this method to initiate the barcode scanning and capture process. Provide the necessary configuration options through the [hvBarcodeConfig] parameter. The result of the scanning process will be returned through the [onComplete] callback function.
  ///
  /// The [hvBarcodeConfig] parameter is optional and allows you to customize the barcode scanning configuration. If not provided, a default [HVBarcodeConfig] instance will be used.
  ///
  /// The [onComplete] callback function takes two parameters: [hvResponse] and [hvError]. [hvResponse] contains the result of the barcode scanning process, while [hvError] represents any errors that occurred during the process. At least one of these parameters will be non-null.
  ///
  /// Example usage:
  /// ```dart
  /// HVBarcodeConfig barcodeConfig = HVBarcodeConfig();
  /// HVBarcodeCapture.start(
  ///   hvBarcodeConfig: barcodeConfig,
  ///   onComplete: (HVResponse? response, HVError? error) {
  ///     if (response != null) {
  ///       // Barcode scanning succeeded, handle the response
  ///     } else if (error != null) {
  ///       // Barcode scanning failed, handle the error
  ///     }
  ///   },
  /// );
  /// ```
  static Future<void> start({
    HVBarcodeConfig? hvBarcodeConfig,
    required void Function(Map?, HVError?) onComplete,
  }) async {
    try {
      final resMap = await _hvBarcodeScanCaptureChannel.invokeMethod(
        HyperSnapSDKConstants.qrScanCaptureStart,
        {
          HyperSnapSDKConstants.hvQRConfig:
              hvBarcodeConfig?.toMap() ?? HVBarcodeConfig().toMap(),
        },
      );

      final errorObj = resMap["errorObj"] as Map;
      final hvError = errorObj.isEmpty ? null : HVError.fromMap(errorObj);
      final resObj = resMap["resultObj"] as Map;

      onComplete(resObj, hvError);
    } catch (e) {
      log(e.toString());
    }
  }
}

/// Configuration options for barcode scanning.
///
/// Use the `HVBarcodeConfig` class to customize the behavior of barcode scanning. You can specify whether to show an instruction page before scanning and provide custom UI strings for the barcode scanning UI.
class HVBarcodeConfig {
  /// Determines whether to show an instruction page before scanning.
  ///
  /// Set this property to `true` if you want to display an instruction page before scanning the barcode. If set to `false`, the scanning process will start immediately without showing any instructions. The default value is `null`.
  final bool? shouldShowInstructionPage;

  /// Custom UI strings for the barcode scanning UI.
  ///
  /// Use this property to provide custom UI strings for the barcode scanning UI. You can localize the text and labels displayed during the scanning process. If not provided, the default UI strings will be used. The default value is `null`.
  final HVBarcodeScanUIStrings? hvBarcodeScanUIStrings;

  /// Creates an instance of [HVBarcodeConfig].
  ///
  /// The [shouldShowInstructionPage] parameter is optional and determines whether to show an instruction page before scanning. The [hvBarcodeScanUIStrings] parameter is also optional and allows you to provide custom UI strings for the barcode scanning UI.
  HVBarcodeConfig({
    this.shouldShowInstructionPage,
    this.hvBarcodeScanUIStrings,
  });

  /// Converts the [HVBarcodeConfig] object to a [Map] representation.
  ///
  /// Use this method to convert the [HVBarcodeConfig] object into a [Map] that can be used for serialization or sending as parameters to the platform-specific code.
  Map<String, dynamic> toMap() {
    return {
      HyperSnapSDKConstants.qrShouldShowInstructionPage:
          shouldShowInstructionPage,
      HyperSnapSDKConstants.qrScanUIStrings: hvBarcodeScanUIStrings?.toMap(),
    };
  }
}

/// UI strings for the barcode scanning UI.
///
/// Use the `HVBarcodeScanUIStrings` class to provide custom text for the barcode scanning user interface. You can customize various strings such as capture titles, descriptions, instructions, and skip text.
class HVBarcodeScanUIStrings {
  /// The title text displayed in the barcode scanning instructions UI.
  ///
  /// Use this property to provide a custom title text that will be displayed in the barcode scanning instructions UI. This can be used to customize the main title of the instructions screen. The default value is `null`.
  final String? barcodeInstructionsTitle;

  /// The description text displayed in the barcode scanning instructions UI.
  ///
  /// Use this property to provide a custom description text that will be displayed in the barcode scanning instructions UI. This can be used to provide additional information or instructions to the user. The default value is `null`.
  final String? barcodeInstructionsDescription;

  /// The text displayed for the "proceed" instruction in the barcode scanning UI.
  ///
  /// Use this property to provide a custom text for the "proceed" instruction in the barcode scanning UI. This text is typically displayed as an action or button text to instruct the user to proceed with the scanning process. The default value is `null`.
  final String? barcodeInstructionsProceed;

  /// The title text displayed in the barcode scanning UI.
  ///
  /// Use this property to provide a custom title text that will be displayed in the barcode scanning UI. This can be used to customize the main title of the scanning screen. The default value is `null`.
  final String? barcodeCaptureTitle;

  /// The description text displayed in the barcode scanning UI.
  ///
  /// Use this property to provide a custom description text that will be displayed in the barcode scanning UI. This can be used to provide additional information or instructions to the user. The default value is `null`.
  final String? barcodeCaptureDescription;

  /// The subtext displayed in the barcode scanning UI.
  ///
  /// Use this property to provide a custom subtext that will be displayed in the barcode scanning UI. This can be used to provide additional information or instructions to the user. The default value is `null`.
  final String? barcodeCaptureSubText;

  /// The text displayed for the "skip" option in the barcode scanning UI.
  ///
  /// Use this property to provide a custom text for the "skip" option in the barcode scanning UI. This text is typically displayed as an option for the user to skip the barcode scanning process. The default value is `null`.
  final String? barcodeCaptureSkip;

  /// The [barcodeCaptureCameraAccessTitle]
  final String? barcodeCaptureCameraAccessTitle;

  /// The [barcodeCaptureCameraAccessDescription]
  final String? barcodeCaptureCameraAccessDescription;

  /// The [barcodeCaptureCameraAccessButton]
  final String? barcodeCaptureCameraAccessButton;

  /// Creates an instance of [HVBarcodeScanUIStrings].
  ///
  /// The [barcodeInstructionsTitle], [barcodeInstructionsDescription], [barcodeInstructionsProceed], [barcodeCaptureTitle], [barcodeCaptureDescription], [barcodeCaptureSubText], [barcodeCaptureSkip], [barcodeCaptureCameraAccessTitle], [barcodeCaptureCameraAccessDescription] and [barcodeCaptureCameraAccessButton] parameters are optional and allow you to provide custom UI strings for the barcode scanning UI.
  HVBarcodeScanUIStrings({
    this.barcodeInstructionsTitle,
    this.barcodeInstructionsDescription,
    this.barcodeInstructionsProceed,
    this.barcodeCaptureTitle,
    this.barcodeCaptureDescription,
    this.barcodeCaptureSubText,
    this.barcodeCaptureSkip,
    this.barcodeCaptureCameraAccessTitle,
    this.barcodeCaptureCameraAccessDescription,
    this.barcodeCaptureCameraAccessButton,
  });

  /// Converts the [HVBarcodeScanUIStrings] object to a [Map] representation.
  ///
  /// Use this method to convert the [HVBarcodeScanUIStrings] object into a [Map] that can be used for serialization or sending as parameters to the platform-specific code.
  Map<String, dynamic> toMap() {
    return {
      HyperSnapSDKConstants.barcodeInstructionsTitle: barcodeInstructionsTitle,
      HyperSnapSDKConstants.barcodeInstructionsDescription: barcodeInstructionsDescription,
      HyperSnapSDKConstants.barcodeInstructionsProceed:
          barcodeInstructionsProceed,
      HyperSnapSDKConstants.barcodeCaptureTitle: barcodeCaptureTitle,
      HyperSnapSDKConstants.barcodeCaptureDescription: barcodeCaptureDescription,
      HyperSnapSDKConstants.barcodeCaptureSubText: barcodeCaptureSubText,
      HyperSnapSDKConstants.barcodeCaptureSkip: barcodeCaptureSkip,
      HyperSnapSDKConstants.barcodeCaptureCameraAccessTitle:
          barcodeCaptureCameraAccessTitle,
      HyperSnapSDKConstants.barcodeCaptureCameraAccessDescription:
          barcodeCaptureCameraAccessDescription,
      HyperSnapSDKConstants.barcodeCaptureCameraAccessButton:
          barcodeCaptureCameraAccessButton,
    };
  }

  /// Creates an instance of [HVBarcodeScanUIStrings] from a map.
  ///
  /// [uiStrings] is a map containing the custom string values.
  static HVBarcodeScanUIStrings fromMap(Map uiStrings) {
    return HVBarcodeScanUIStrings(
      barcodeInstructionsTitle: uiStrings["qrInstructions_title"],
      barcodeInstructionsDescription: uiStrings["qrInstructions_desc"],
      barcodeInstructionsProceed: uiStrings["qrInstructions_button"],
      barcodeCaptureTitle: uiStrings["qrCapture_title"],
      barcodeCaptureDescription: uiStrings["qrCapture_desc"],
      barcodeCaptureSubText: uiStrings["qrCapture_subText"],
      barcodeCaptureSkip: uiStrings["qrCapture_skipText"],
      barcodeCaptureCameraAccessTitle: uiStrings["qrCapture_cameraAccessTitle"],
      barcodeCaptureCameraAccessDescription: uiStrings["qrCapture_cameraAccessDesc"],
      barcodeCaptureCameraAccessButton: uiStrings["qrCapture_cameraAccessButton"],
    );
  }
}
