import 'dart:developer';

import 'package:flutter/services.dart';

import 'hv_result.dart';
import 'hyper_snap_sdk_constants.dart';

/// An enumeration representing the side of a document.
///
/// Use the [DocumentSide] enum to indicate whether a document is the front side or the back side.
enum DocumentSide {
  /// The front side of the document.
  front,

  /// The back side of the document.
  back
}

/// Extension methods for the [DocumentSide] enum.
extension DocumentSideExtension on DocumentSide {
  /// Returns a string representation of the document side.
  ///
  /// The returned string will be one of the following values: 'FRONT' for the front side, 'BACK' for the back side.
  String get documentSideString {
    switch (this) {
      case DocumentSide.front:
        return 'FRONT';
      case DocumentSide.back:
        return 'BACK';
      default:
        return 'FRONT';
    }
  }
}

/// An enumeration representing the type of a document.
///
/// Use the [DocumentType] enum to specify the type of a document, such as a card, A4 paper, passport, or other types.
enum DocumentType {
  /// A card-type document.
  card,

  /// An A4-sized document.
  a4,

  /// A passport document.
  passport,

  /// Another type of document.
  other
}

/// Extension methods for the [DocumentType] enum.
extension DocumentTypeExtension on DocumentType {
  /// Returns a string representation of the document type.
  ///
  /// The returned string will be one of the following values: 'CARD' for a card-type document,
  /// 'A4' for an A4-sized document, 'PASSPORT' for a passport document, 'OTHER' for other types of documents.
  String get documentTypeString {
    switch (this) {
      case DocumentType.card:
        return 'CARD';
      case DocumentType.a4:
        return 'A4';
      case DocumentType.passport:
        return 'PASSPORT';
      case DocumentType.other:
        return 'OTHER';
      default:
        return 'CARD';
    }
  }
}

/// The class that provides methods for document capture using HyperSnap SDK.
class HVDocsCapture {
  static const MethodChannel _hyperSnapSDKChannel =
      MethodChannel(HyperSnapSDKConstants.hyperSnapChannel);

  /// Starts the document capture process and retrieves the parsed result.
  ///
  /// Parameters:
  ///   - hvDocConfig (optional): An instance of [HVDocConfig] representing the document capture configuration.
  ///   - onComplete: A callback function of type [Function(HVResponse?, HVError?)] that will be called when the capture process is complete.
  ///
  /// The `onComplete` function should handle the resulting [HVResponse] and [HVError].
  ///
  /// Example usage:
  /// ```dart
  /// HVDocsCapture.start(
  ///   hvDocConfig: HVDocConfig(),
  ///   onComplete: (response, error) {
  ///     // Handle the response and error here
  ///   },
  /// );
  /// ```
  static Future<void> start({
    HVDocConfig? hvDocConfig,
    required Function(HVResponse?, HVError?) onComplete,
  }) async {
    try {
      final resMap = await _hyperSnapSDKChannel.invokeMethod(
        HyperSnapSDKConstants.startDocCapture,
        {
          HyperSnapSDKConstants.docConfig:
              hvDocConfig?.toMap() ?? HVDocConfig().toMap(),
        },
      );

      final errorObj = resMap["errorObj"];
      final resObj = resMap["resultObj"];

      final hvError = (errorObj.isEmpty) ? null : HVError.fromMap(errorObj);
      final hvResponse = (resObj.isEmpty) ? null : HVResponse.fromMap(resObj);

      onComplete(hvResponse, hvError);
    } catch (e) {
      log(e.toString());
    }
  }
}

/// HVDocConfig - used to configure the doc capture options for the sdk
/// HVDocConfig is used to configure the document capture options for the SDK.
class HVDocConfig {
  ///   hvDocCaptureUIStrings:  The document capture UI strings. Type: `HVDocCaptureUIStrings`.
  HVDocCaptureUIStrings? hvDocCaptureUIStrings;

  ///   shouldShowTrustLogos: Specifies whether to show trust logos in HyperSnapSDK screens. Type: `bool`.
  bool? shouldShowTrustLogos;

  ///   shouldShowInstructionPage: Specifies whether to show the instructions page to the user. Type: `bool`.
  bool? shouldShowInstructionPage;

  ///   shouldEnableDocumentUpload: Enables the document upload option. Type: `bool`.
  bool? shouldEnableDocumentUpload;

  ///   shouldShowFlashIcon: Specifies whether to display the flash icon to the user. Type: `bool`.
  bool? shouldShowFlashIcon;

  ///   docType: The type of the document to be captured. Type: `DocumentType`.
  DocumentType? docType;

  ///   aspectRatioValue: The aspect ratio of the document to be captured. Type: `double`.
  double? aspectRatioValue;

  ///   shouldAddPadding: Specifies whether the captured image needs to be padded. Type: `bool`.
  bool? shouldAddPadding;

  ///   padding: The padding value for document captures. Type: `double`.
  double? padding;

  ///   shouldShowCloseAlert: Specifies whether to display the close alert dialog when the user tries to close any HyperSnapSDK screens. Type: `bool`.
  bool? shouldShowCloseAlert;

  ///   closeAlertDialogTitle: The custom title for the alert dialog that appears when closing any screen. Type: `String`.
  String? closeAlertDialogTitle;

  ///   closeAlertDialogDescription: The custom description for the alert dialog that appears when closing any screen. Type: `String`.  ///
  String? closeAlertDialogDescription;

  ///   shouldAutoCapture: Specifies whether to enable auto capture functionality. Type: `bool`.
  ///   Please contact Hyperverge incase you want to enable this feature
  bool? shouldAutoCapture;

  ///   autoCaptureDuration: The wait duration for auto capture. Type: `int`.
  ///   Please contact Hyperverge incase you want to enable this feature
  int? autoCaptureDuration;

  ///   shouldReadBarcode: Specifies whether to read barcode in document capture screen. Type: `bool`.
  bool? shouldReadBarcode;

  ///   readBarcodeTimeout: The duration after which the capture button is enabled when `readBarcode` is enabled. Type: `int`.
  int? readBarcodeTimeout;

  ///   hvOCRAPIDetails: The OCR API details. Type: `HVOCRAPIDetails`.
  HVOCRAPIDetails? hvOCRAPIDetails;

  ///   shouldShowReviewScreen: Specifies whether to show the review screen to the user. Type: `bool`.
  bool? shouldShowReviewScreen;

  ///   shouldEnableRetries: Specifies whether the SDK should handle retries after any document capture screen. Type: `bool`.
  bool? shouldEnableRetries;

  /// Constructs a new [HVDocConfig] object with the specified configuration options.
  HVDocConfig({
    this.hvDocCaptureUIStrings,
    this.shouldShowTrustLogos,
    this.shouldShowInstructionPage,
    this.shouldEnableDocumentUpload,
    this.shouldShowFlashIcon,
    this.docType,
    this.aspectRatioValue,
    this.shouldAddPadding,
    this.padding,
    this.shouldShowCloseAlert,
    this.closeAlertDialogTitle,
    this.closeAlertDialogDescription,
    this.shouldAutoCapture,
    this.autoCaptureDuration,
    this.shouldReadBarcode,
    this.readBarcodeTimeout,
    this.hvOCRAPIDetails,
    this.shouldShowReviewScreen,
    this.shouldEnableRetries,
  });

  /// Converts the [HVDocConfig] object to a map of key-value pairs.
  ///
  /// Returns a map representing the [HVDocConfig] object.
  Map<String, dynamic> toMap() {
    return {
      HyperSnapSDKConstants.docCaptureUIStrings: hvDocCaptureUIStrings?.toMap(),
      HyperSnapSDKConstants.docShouldShowTrustLogos: shouldShowTrustLogos,
      HyperSnapSDKConstants.docShouldShowInstructionPage:
          shouldShowInstructionPage,
      HyperSnapSDKConstants.docShouldEnableUpload: shouldEnableDocumentUpload,
      HyperSnapSDKConstants.docShouldShowFlashIcon: shouldShowFlashIcon,
      HyperSnapSDKConstants.docTypeValue: docType?.documentTypeString,
      HyperSnapSDKConstants.docAspectRatioValue: aspectRatioValue,
      HyperSnapSDKConstants.docShouldSetPadding: shouldAddPadding,
      HyperSnapSDKConstants.docDefaultPaddingValue: padding,
      HyperSnapSDKConstants.docShouldShowCloseAlert: shouldShowCloseAlert,
      HyperSnapSDKConstants.docCloseAlertDialogTitle: closeAlertDialogTitle,
      HyperSnapSDKConstants.docCloseAlertDialogDescription:
          closeAlertDialogDescription,
      HyperSnapSDKConstants.docAutoCapture: shouldAutoCapture,
      HyperSnapSDKConstants.docAutoCaptureDuration: autoCaptureDuration,
      HyperSnapSDKConstants.docShouldReadBarcode: shouldReadBarcode,
      HyperSnapSDKConstants.docReadBarcodeTimeout: readBarcodeTimeout,
      HyperSnapSDKConstants.ocrApiDetails: hvOCRAPIDetails?.toMap(),
      HyperSnapSDKConstants.docShouldShowReviewScreen: shouldShowReviewScreen,
      HyperSnapSDKConstants.docShouldEnableRetries: shouldEnableRetries,
    };
  }
}

/// HVOCRAPIDetails represents the details of the OCR API for document capture.
class HVOCRAPIDetails {
  /// The [ocrEndpoint] specifies the OCR API endpoint.
  String ocrEndpoint;

  /// The [documentSide] specifies the document side to be processed.
  DocumentSide documentSide;

  /// The [params] specify additional parameters for the OCR API.
  Map? params;

  /// The [headers] specify additional headers for the OCR API.
  Map? headers;

  /// Constructs a new [HVOCRAPIDetails] object with the specified API details.
  HVOCRAPIDetails({
    required this.ocrEndpoint,
    required this.documentSide,
    this.params,
    this.headers,
  });

  /// Converts the [HVOCRAPIDetails] object to a map of key-value pairs.
  ///
  /// Returns a map representing the [HVOCRAPIDetails] object.
  Map<String, dynamic> toMap() {
    return {
      HyperSnapSDKConstants.ocrEndpoint: ocrEndpoint,
      HyperSnapSDKConstants.documentSideString: documentSide.documentSideString,
      HyperSnapSDKConstants.params: params,
      HyperSnapSDKConstants.headers: headers,
    };
  }
}

/// HVDocCaptureUIStrings represents the localized UI strings for the document capture user interface.
class HVDocCaptureUIStrings {
  /// The [docInstructionsTitle] represents the title for document capture instructions.
  String? docInstructionsTitle;

  /// The [docInstructionsDescription] represents the description for document capture instructions.
  String? docInstructionsDescription;

  /// The [docInstructionsProceed] represents the instruction to proceed with document capture.
  String? docInstructionsProceed;

  /// The [docInstructionsUploadButton] represents the instruction to proceed with document upload.
  String? docInstructionsUploadButton;

  /// The [docCaptureTitle] represents the title for the document capture screen.
  String? docCaptureTitle;

  /// The [docCaptureDescription] represents the description for the document capture screen.
  String? docCaptureDescription;

  /// The [docCaptureSubText] represents the subtext for the document capture screen.
  String? docCaptureSubText;

  /// The [docCaptureScanningBarcode] represents the text shown when the user is trying to read the barcode in the document capture screen.
  String? docCaptureScanningBarcode;

  /// The [docCaptureBarcodeScanned] represents the text shown when the user has read the barcode in the document capture screen.
  String? docCaptureBarcodeScanned;

  /// The [docCaptureDocFound] represents the text shown when the document is found in the camera preview in the document capture screen.
  String? docCaptureDocFound;

  /// The [docCaptureDocNotFound] represents the text shown when the document is not found in the camera preview in the document capture screen.
  String? docCaptureDocNotFound;

  /// The [docCaptureMoveCloser] represents the text shown when the document is zoomed out in the camera preview in the document capture screen.
  String? docCaptureMoveCloser;

  /// The [docCaptureMoveAwayFromCamera] represents the text shown when the document is zoomed in in the camera preview in the document capture screen.
  String? docCaptureMoveAwayFromCamera;

  /// The [docCaptureMoveAwayFromEdge] represents the text shown when the document is close to the edge of the camera preview in the document capture screen.
  String? docCaptureMoveAwayFromEdge;

  /// The [docCaptureAutoCaptureWait] represents the text shown when the document is going to be auto-captured in the document capture screen.
  String? docCaptureAutoCaptureWait;

  /// The [docCaptureAutoCaptureError] represents the text shown when the captured document does not satisfy the requirements - shown in review screen.
  String? docCaptureAutoCaptureError;

  /// The [docCaptureCameraAccessTitle]
  String? docCaptureCameraAccessTitle;

  /// The [docCaptureCameraAccessDescription]
  String? docCaptureCameraAccessDescription;

  /// The [docCaptureCameraAccessButton]
  String? docCaptureCameraAccessButton;

  /// The [docReviewTitle] represents the title for the document review screen.
  String? docReviewTitle;

  /// The [docReviewDescription] represents the description for the document review screen.
  String? docReviewDescription;

  /// The [docReviewContinueButton] represents the label for the continue button on the document review screen.
  String? docReviewContinueButton;

  /// The [docReviewRetakeButton] represents the label for the retake button on the document review screen.
  String? docReviewRetakeButton;

  /// The [docLoaderTitle] represents the title for the document loading screen.
  String? docLoaderTitle;

  /// The [docLoaderDescription] represents the description for the document loading screen.
  String? docLoaderDescription;

  /// The [docRetakeTitle] represents the title text for the retake screen.
  String? docRetakeTitle;

  /// The [docRetakeButton] represents the label for the retake button on the retake screen.
  String? docRetakeButton;

  /// Constructs a new [HVDocCaptureUIStrings] object with the specified UI strings.
  HVDocCaptureUIStrings({
    this.docInstructionsTitle,
    this.docInstructionsDescription,
    this.docInstructionsProceed,
    this.docInstructionsUploadButton,
    this.docCaptureTitle,
    this.docCaptureDescription,
    this.docCaptureSubText,
    this.docCaptureScanningBarcode,
    this.docCaptureBarcodeScanned,
    this.docCaptureDocFound,
    this.docCaptureDocNotFound,
    this.docCaptureMoveCloser,
    this.docCaptureMoveAwayFromCamera,
    this.docCaptureMoveAwayFromEdge,
    this.docCaptureAutoCaptureWait,
    this.docCaptureAutoCaptureError,
    this.docCaptureCameraAccessTitle,
    this.docCaptureCameraAccessDescription,
    this.docCaptureCameraAccessButton,
    this.docReviewTitle,
    this.docReviewDescription,
    this.docReviewContinueButton,
    this.docReviewRetakeButton,
    this.docLoaderTitle,
    this.docLoaderDescription,
    this.docRetakeTitle,
    this.docRetakeButton,
  });

  /// Converts the [HVDocCaptureUIStrings] object to a map of key-value pairs.
  ///
  /// Returns a map representing the [HVDocCaptureUIStrings] object.
  Map<String, dynamic> toMap() {
    return {
      HyperSnapSDKConstants.docInstructionsTitle: docInstructionsTitle,
      HyperSnapSDKConstants.docInstructionsDescription:
          docInstructionsDescription,
      HyperSnapSDKConstants.docInstructionsProceed: docInstructionsProceed,
      HyperSnapSDKConstants.docInstructionsUploadButton:
          docInstructionsUploadButton,
      HyperSnapSDKConstants.docCaptureTitle: docCaptureTitle,
      HyperSnapSDKConstants.docCaptureDescription: docCaptureDescription,
      HyperSnapSDKConstants.docCaptureSubText: docCaptureSubText,
      HyperSnapSDKConstants.docCaptureScanningBarcode:
          docCaptureScanningBarcode,
      HyperSnapSDKConstants.docCaptureBarcodeScanned: docCaptureBarcodeScanned,
      HyperSnapSDKConstants.docCaptureDocFound: docCaptureDocFound,
      HyperSnapSDKConstants.docCaptureDocNotFound: docCaptureDocNotFound,
      HyperSnapSDKConstants.docCaptureMoveCloser: docCaptureMoveCloser,
      HyperSnapSDKConstants.docCaptureMoveAwayFromCamera:
          docCaptureMoveAwayFromCamera,
      HyperSnapSDKConstants.docCaptureMoveAwayFromEdge:
          docCaptureMoveAwayFromEdge,
      HyperSnapSDKConstants.docCaptureAutoCaptureWait:
          docCaptureAutoCaptureWait,
      HyperSnapSDKConstants.docCaptureAutoCaptureError:
          docCaptureAutoCaptureError,
      HyperSnapSDKConstants.docCaptureCameraAccessTitle:
          docCaptureCameraAccessTitle,
      HyperSnapSDKConstants.docCaptureCameraAccessDesc:
          docCaptureCameraAccessDescription,
      HyperSnapSDKConstants.docCaptureCameraAccessButton:
          docCaptureCameraAccessButton,
      HyperSnapSDKConstants.docReviewTitle: docReviewTitle,
      HyperSnapSDKConstants.docReviewDescription: docReviewDescription,
      HyperSnapSDKConstants.docReviewContinueButton: docReviewContinueButton,
      HyperSnapSDKConstants.docReviewRetakeButton: docReviewRetakeButton,
      HyperSnapSDKConstants.docLoaderTitle: docLoaderTitle,
      HyperSnapSDKConstants.docLoaderDesc: docLoaderDescription,
      HyperSnapSDKConstants.docRetakeTitle: docRetakeTitle,
      HyperSnapSDKConstants.docRetakeButton: docRetakeButton,
    };
  }

  /// Constructs a [HVDocCaptureUIStrings] object from a map of key-value pairs.
  ///
  /// The [uiStrings] map contains the UI strings for the [HVDocCaptureUIStrings] object.
  ///
  /// Returns a [HVDocCaptureUIStrings] object constructed from the map.
  static HVDocCaptureUIStrings fromMap(Map uiStrings) {
    return HVDocCaptureUIStrings(
      docInstructionsTitle: uiStrings["docInstructions_title"],
      docInstructionsDescription: uiStrings["docInstructions_desc"],
      docInstructionsProceed: uiStrings["docInstructions_button"],
      docInstructionsUploadButton: uiStrings["docInstructions_uploadButton"],
      docCaptureTitle: uiStrings["docCapture_title"],
      docCaptureDescription: uiStrings["docCapture_desc"],
      docCaptureSubText: uiStrings["docCaptureSubText"],
      docCaptureScanningBarcode: uiStrings["docCapture_scanningBarcode"],
      docCaptureBarcodeScanned: uiStrings["docCapture_barcodeScanned"],
      docCaptureDocFound: uiStrings["docCapture_docFound"],
      docCaptureDocNotFound: uiStrings["docCapture_docNotFound"],
      docCaptureMoveCloser: uiStrings["docCapture_moveCloser"],
      docCaptureMoveAwayFromCamera: uiStrings["docCapture_moveAwayFromCamera"],
      docCaptureMoveAwayFromEdge: uiStrings["docCapture_moveAwayFromEdge"],
      docCaptureAutoCaptureWait: uiStrings["docCapture_autoCaptureWait"],
      docCaptureAutoCaptureError: uiStrings["docCapture_autoCaptureError"],
      docCaptureCameraAccessTitle: uiStrings["docCapture_cameraAccessTitle"],
      docCaptureCameraAccessDescription:
          uiStrings["docCapture_cameraAccessDesc"],
      docCaptureCameraAccessButton: uiStrings["docCapture_cameraAccessButton"],
      docReviewTitle: uiStrings["docReview_title"],
      docReviewDescription: uiStrings["docReview_desc"],
      docReviewContinueButton: uiStrings["docReview_usePhotoButton"],
      docReviewRetakeButton: uiStrings["docReview_retakeButton"],
      docLoaderTitle: uiStrings["docLoader_title"],
      docLoaderDescription: uiStrings["docLoader_desc"],
      docRetakeTitle: uiStrings["docRetake_title"],
      docRetakeButton: uiStrings["docRetake_button"],
    );
  }
}
