import 'dart:developer';

import 'package:flutter/services.dart';

import 'hv_result.dart';
import 'hyper_snap_sdk_constants.dart';

/// Enum representing different liveness modes.
enum LivenessMode {
  /// No liveness mode.
  none,

  /// Texture liveness mode.
  textureLiveness,
}

/// Extension methods for the [LivenessMode] enum.
extension LivenessModeExtension on LivenessMode {
  /// Returns the string representation of the liveness mode.
  ///
  /// Returns 'NONE' for [LivenessMode.none] and 'TEXTURELIVENESS' for [LivenessMode.textureLiveness].
  String get livenessModeString {
    switch (this) {
      case LivenessMode.none:
        return 'NONE';
      case LivenessMode.textureLiveness:
        return 'TEXTURELIVENESS';
      default:
        return 'TEXTURELIVENESS';
    }
  }
}

/// Class that provides face capture functionality.
class HVFaceCapture {
  static const MethodChannel _hyperSnapSDKChannel =
      MethodChannel(HyperSnapSDKConstants.hyperSnapChannel);

  /// Starts the face capture process and returns the parsed result.
  ///
  /// [hvFaceConfig] (optional) is an instance of [HVFaceConfig] that configures the face capture.
  ///
  /// [onComplete] is a callback function that receives the [HVResponse] and [HVError] objects.
  ///
  /// Example usage:
  /// ```dart
  /// HVFaceCapture.start(
  ///   hvFaceConfig: HVFaceConfig(), // Provide the face capture configuration if needed
  ///   onComplete: (response, error) {
  ///     // Handle the response and error objects
  ///   },
  /// );
  /// ```
  static Future<void> start({
    HVFaceConfig? hvFaceConfig,
    required void Function(HVResponse?, HVError?) onComplete,
  }) async {
    Map resMap = {};
    HVError? hvError;
    HVResponse? hvResponse;

    try {
      resMap = await _hyperSnapSDKChannel.invokeMethod(
        HyperSnapSDKConstants.startFaceCapture,
        {
          HyperSnapSDKConstants.faceConfig:
              hvFaceConfig?.toMap() ?? HVFaceConfig().toMap(),
        },
      );

      Map errorObj = resMap["errorObj"];
      Map resObj = resMap["resultObj"];
      hvError = (errorObj.isEmpty) ? null : HVError.fromMap(errorObj);
      hvResponse = (resObj.isEmpty) ? null : HVResponse.fromMap(resObj);
    } catch (e) {
      log(e.toString());
    }

    onComplete(hvResponse, hvError);
  }
}

/// HVFaceConfig is used to customise HyperSnapSDK face capture flow.
class HVFaceConfig {
  HVFaceCaptureUIStrings? hvFaceCaptureUIStrings;
  bool? shouldShowTrustLogos;
  bool? shouldShowInstructionPage;
  bool? shouldShowCameraSwitchButton;
  bool? shouldUseBackCamera;
  bool? shouldAddPadding;
  HVFaceCapturePadding? padding;
  bool? shouldShowCloseAlert;
  String? closeAlertDialogTitle;
  String? closeAlertDialogDescription;
  bool? shouldAutoCapture;
  int? autoCaptureDuration;
  HVVideoRecordingConfig? videoRecordingConfig;
  bool? shouldCheckActiveLiveness;
  int? totalGestures;
  HVLivenessAPIDetails? hvLivenessAPIDetails;
  bool? shouldReturnFullImageUrl;
  bool? shouldEnableRetries;

  /// Creates an instance of [HVFaceConfig] with the specified configuration.
  HVFaceConfig({
    this.hvFaceCaptureUIStrings,
    this.shouldShowTrustLogos,
    this.shouldShowInstructionPage,
    this.shouldShowCameraSwitchButton,
    this.shouldUseBackCamera,
    this.shouldAddPadding,
    this.padding,
    this.shouldShowCloseAlert,
    this.closeAlertDialogTitle,
    this.closeAlertDialogDescription,
    this.shouldAutoCapture,
    this.autoCaptureDuration,
    this.videoRecordingConfig,
    this.shouldCheckActiveLiveness,
    this.totalGestures,
    this.hvLivenessAPIDetails,
    this.shouldReturnFullImageUrl,
    this.shouldEnableRetries,
  });

  /// Converts the [HVFaceConfig] object to a map.
  ///
  /// Returns a map containing the hvFaceConfig values.
  Map<String, dynamic> toMap() {
    return {
      HyperSnapSDKConstants.faceCaptureUIStrings:
          hvFaceCaptureUIStrings?.toMap(),
      HyperSnapSDKConstants.faceShouldShowTrustLogos: shouldShowTrustLogos,
      HyperSnapSDKConstants.faceShouldShowInstructionPage:
          shouldShowInstructionPage,
      HyperSnapSDKConstants.faceShouldShowCameraSwitchButton:
          shouldShowCameraSwitchButton,
      HyperSnapSDKConstants.faceShouldUseBackCamera: shouldUseBackCamera,
      HyperSnapSDKConstants.faceShouldAddPadding: shouldAddPadding,
      HyperSnapSDKConstants.facePadding: padding?.toMap(),
      HyperSnapSDKConstants.faceShouldShowCloseAlert: shouldShowCloseAlert,
      HyperSnapSDKConstants.faceCloseAlertDialogTitle: closeAlertDialogTitle,
      HyperSnapSDKConstants.faceCloseAlertDialogDesc:
          closeAlertDialogDescription,
      HyperSnapSDKConstants.faceAutoCapture: shouldAutoCapture,
      HyperSnapSDKConstants.autoCaptureDuration: autoCaptureDuration,
      HyperSnapSDKConstants.faceVideoRecording: videoRecordingConfig?.toMap(),
      HyperSnapSDKConstants.checkActiveLiveness: shouldCheckActiveLiveness,
      HyperSnapSDKConstants.totalGestures: totalGestures,
      HyperSnapSDKConstants.livenessApiDetails: hvLivenessAPIDetails?.toMap(),
      HyperSnapSDKConstants.faceShouldReturnFullImageUrl:
          shouldReturnFullImageUrl,
      HyperSnapSDKConstants.faceShouldHandleRetries: shouldEnableRetries,
    };
  }
}

/// HVLivenessAPIDetails is used to pre-set the endpoint, liveness value, params, and headers for making a liveness API call immediately after the document has been captured.
class HVLivenessAPIDetails {
  String? livenessEndpoint;
  LivenessMode? livenessMode;
  Map? params;
  Map? headers;

  /// Creates an instance of [HVLivenessAPIDetails] to pre-set the endpoint, liveness value, params, and headers for making a liveness API call.
  ///
  /// [livenessEndpoint] (optional) is the endpoint URL which will be used to make the liveness call. By default, the SDK provides a default endpoint for making liveness calls.
  /// [livenessMode] (optional) is the liveness mode, which can be one of [LivenessMode.none, LivenessMode.textureLiveness], specifying the side of the document captured.
  /// [params] (optional) is a map/json containing key-value pairs of different fraud/malice checks provided by Hyperverge.co.
  /// [headers] (optional) is a map/json containing key-value pairs of different header options provided by Hyperverge.co.
  HVLivenessAPIDetails({
    this.livenessEndpoint,
    this.livenessMode,
    this.params,
    this.headers,
  });

  /// Converts the [HVLivenessAPIDetails] object to a map.
  ///
  /// Returns a map containing the pre-set values.
  Map<String, dynamic> toMap() {
    return {
      HyperSnapSDKConstants.livenessEndpoint: livenessEndpoint,
      HyperSnapSDKConstants.livenessValue: livenessMode?.livenessModeString,
      HyperSnapSDKConstants.livenessParams: params,
      HyperSnapSDKConstants.livenessHeaders: headers,
    };
  }
}

/// HVFaceCaptureUIStrings is used to set custom strings for all customizable text in HyperSnapSDK face capture.
class HVFaceCaptureUIStrings {
  String? faceInstructionsTitle;
  String? faceInstructionsDescription;
  String? faceInstructionsProceed;
  String? faceInstructionsProceedBackCam;
  String? faceCaptureTitle;
  String? faceCaptureFaceFound;
  String? faceCaptureFaceNotFound;
  String? faceCaptureMoveCloser;
  String? faceCaptureMoveAway;
  String? faceCaptureLookStraight;
  String? faceCaptureMultipleFaces;
  String? faceCaptureDeviceWrongOrientation;
  String? faceCaptureAutoCaptureWait;
  String? faceCaptureAutoCaptureAction;
  String? faceGestureLookLeft;
  String? faceGestureLookStraight;
  String? faceGestureLookRight;
  String? faceCaptureCameraAccessTitle;
  String? faceCaptureCameraAccessDescription;
  String? faceCaptureCameraAccessButton;
  String? faceLoaderTitle;
  String? faceLoaderDescription;
  String? faceRetakeTitle;
  String? faceRetakeButton;

  /// Creates an instance of [HVFaceCaptureUIStrings] to set custom strings for all customizable text in HyperSnapSDK face capture.
  HVFaceCaptureUIStrings({
    this.faceInstructionsTitle,
    this.faceInstructionsDescription,
    this.faceInstructionsProceed,
    this.faceInstructionsProceedBackCam,
    this.faceCaptureTitle,
    this.faceCaptureFaceFound,
    this.faceCaptureFaceNotFound,
    this.faceCaptureMoveCloser,
    this.faceCaptureMoveAway,
    this.faceCaptureLookStraight,
    this.faceCaptureMultipleFaces,
    this.faceCaptureDeviceWrongOrientation,
    this.faceCaptureAutoCaptureWait,
    this.faceCaptureAutoCaptureAction,
    this.faceGestureLookLeft,
    this.faceGestureLookRight,
    this.faceGestureLookStraight,
    this.faceCaptureCameraAccessTitle,
    this.faceCaptureCameraAccessDescription,
    this.faceCaptureCameraAccessButton,
    this.faceLoaderTitle,
    this.faceLoaderDescription,
    this.faceRetakeTitle,
    this.faceRetakeButton,
  });

  /// Converts the [HVFaceCaptureUIStrings] object to a map.
  ///
  /// Returns a map containing the custom string values.
  Map<String, dynamic> toMap() {
    return {
      HyperSnapSDKConstants.faceInstructionsTitle: faceInstructionsTitle,
      HyperSnapSDKConstants.faceInstructionsDesc: faceInstructionsDescription,
      HyperSnapSDKConstants.faceInstructionsProceed: faceInstructionsProceed,
      HyperSnapSDKConstants.faceInstructionsProceedBackCam:
          faceInstructionsProceedBackCam,
      HyperSnapSDKConstants.faceCaptureTitle: faceCaptureTitle,
      HyperSnapSDKConstants.faceCaptureMoveCloser: faceCaptureMoveCloser,
      HyperSnapSDKConstants.faceCaptureFaceFound: faceCaptureFaceFound,
      HyperSnapSDKConstants.faceCaptureFaceNotFound: faceCaptureFaceNotFound,
      HyperSnapSDKConstants.faceCaptureMoveAway: faceCaptureMoveAway,
      HyperSnapSDKConstants.faceCaptureLookStraight: faceCaptureLookStraight,
      HyperSnapSDKConstants.faceCaptureMultipleFaces: faceCaptureMultipleFaces,
      HyperSnapSDKConstants.faceCaptureDeviceWrongOrientation:
          faceCaptureDeviceWrongOrientation,
      HyperSnapSDKConstants.faceCaptureAutoCaptureWait:
          faceCaptureAutoCaptureWait,
      HyperSnapSDKConstants.faceCaptureAutoCaptureAction:
          faceCaptureAutoCaptureAction,
      HyperSnapSDKConstants.faceGestureLookLeft: faceGestureLookLeft,
      HyperSnapSDKConstants.faceGestureLookRight: faceGestureLookRight,
      HyperSnapSDKConstants.faceGestureLookStraight: faceGestureLookStraight,
      HyperSnapSDKConstants.faceCaptureCameraAccessTitle:
          faceCaptureCameraAccessTitle,
      HyperSnapSDKConstants.faceCaptureCameraAccessDesc:
          faceCaptureCameraAccessDescription,
      HyperSnapSDKConstants.faceCaptureCameraAccessButton:
          faceCaptureCameraAccessButton,
      HyperSnapSDKConstants.faceLoaderDesc: faceLoaderDescription,
      HyperSnapSDKConstants.faceLoaderTitle: faceLoaderTitle,
      HyperSnapSDKConstants.faceRetakeButton: faceRetakeButton,
      HyperSnapSDKConstants.faceRetakeTitle: faceRetakeTitle,
    };
  }

  /// Creates an instance of [HVFaceCaptureUIStrings] from a map.
  ///
  /// [uiStrings] is a map containing the custom string values.
  static HVFaceCaptureUIStrings fromMap(Map uiStrings) {
    return HVFaceCaptureUIStrings(
      faceInstructionsTitle: uiStrings["faceInstructions_title"],
      faceInstructionsDescription: uiStrings["faceInstructions_desc"],
      faceInstructionsProceed: uiStrings["faceInstructionsProceed"],
      faceInstructionsProceedBackCam:
          uiStrings["faceInstructionsProceedBackCam"],
      faceCaptureTitle: uiStrings["faceCapture_title"],
      faceCaptureFaceFound: uiStrings["faceCapture_faceFound"],
      faceCaptureFaceNotFound: uiStrings["faceCapture_faceNotFound"],
      faceCaptureMoveCloser: uiStrings["faceCapture_moveCloser"],
      faceCaptureMoveAway: uiStrings["faceCapture_moveAway"],
      faceCaptureLookStraight: uiStrings["faceCapture_lookStraight"],
      faceCaptureMultipleFaces: uiStrings["faceCapture_multipleFaces"],
      faceCaptureDeviceWrongOrientation:
          uiStrings["faceCapture_deviceWrongOrientation"],
      faceGestureLookLeft: uiStrings["faceGesture_lookLeft"],
      faceGestureLookStraight: uiStrings["faceGesture_lookStraight"],
      faceGestureLookRight: uiStrings["faceGesture_lookRight"],
      faceCaptureAutoCaptureWait: uiStrings["faceAutoCapture_stayStill"],
      faceCaptureAutoCaptureAction: uiStrings["faceAutoCapture_labelChange"],
      faceCaptureCameraAccessTitle: uiStrings["faceCapture_cameraAccessTitle"],
      faceCaptureCameraAccessDescription: uiStrings["faceCapture_cameraAccessDesc"],
      faceCaptureCameraAccessButton:
          uiStrings["faceCapture_cameraAccessButton"],
      faceLoaderTitle: uiStrings["faceLoader_title"],
      faceLoaderDescription: uiStrings["faceLoader_desc"],
      faceRetakeTitle: uiStrings["faceRetake_title"],
      faceRetakeButton: uiStrings["faceRetake_button"],
    );
  }
}

/// Configuration for video recording in the user's selfie flow.
class HVVideoRecordingConfig {
  /// Determines whether video recording should be enabled.
  final bool shouldRecordVideo;

  /// The number of frames to be recorded.
  final int? noOfFrames;

  /// The bitrate (in million) of the recorded video.
  final double? bitRateM;

  /// The frames per second (FPS) at which the video should be recorded.
  final int? fps;

  /// Creates an instance of [HVVideoRecordingConfig] to configure video recording in the user's selfie flow.
  ///
  /// [shouldRecordVideo] determines whether video recording should be enabled.
  /// [noOfFrames] defines the number of frames to be recorded. Default is 200 frames.
  /// [bitRateM] specifies the bitrate (in million) of the recorded video. Default is 3M bitrate.
  /// [fps] sets the frames per second (FPS) at which the video should be recorded. Default is 30 FPS.
  HVVideoRecordingConfig({
    required this.shouldRecordVideo,
    this.noOfFrames,
    this.bitRateM,
    this.fps,
  });

  /// Converts the [HVVideoRecordingConfig] object to a map.
  ///
  /// Returns a map containing the video recording configuration.
  Map<String, dynamic> toMap() {
    return {
      HyperSnapSDKConstants.noOfFrames: noOfFrames,
      HyperSnapSDKConstants.fps: fps,
      HyperSnapSDKConstants.shouldRecordVideo: shouldRecordVideo,
      HyperSnapSDKConstants.bitRateM: bitRateM,
    };
  }
}

/// Represents the padding values for face capture.
class HVFaceCapturePadding {
  /// The amount of padding on the left side.
  final double left;

  /// The amount of padding on the top side.
  final double top;

  /// The amount of padding on the right side.
  final double right;

  /// The amount of padding on the bottom side.
  final double bottom;

  /// Creates an instance of [HVFaceCapturePadding] with the specified padding values.
  ///
  /// [left] is the amount of padding on the left side.
  /// [top] is the amount of padding on the top side.
  /// [right] is the amount of padding on the right side.
  /// [bottom] is the amount of padding on the bottom side.
  HVFaceCapturePadding({
    this.left = 0.0,
    this.top = 0.0,
    this.right = 0.0,
    this.bottom = 0.0,
  });

  /// Converts the [HVFaceCapturePadding] object to a map.
  ///
  /// Returns a map containing the padding values.
  Map<String, dynamic> toMap() {
    return {
      HyperSnapSDKConstants.paddingLeft: left,
      HyperSnapSDKConstants.paddingTop: top,
      HyperSnapSDKConstants.paddingRight: right,
      HyperSnapSDKConstants.paddingBottom: bottom,
    };
  }
}
