import 'dart:developer';

import 'package:flutter/services.dart';
import 'package:hypersnapsdk_flutter/hv_result.dart';
import 'package:hypersnapsdk_flutter/hyper_snap_sdk_constants.dart';

/// FaceMatchMode is an enumeration that represents the different modes for performing face matching.
enum FaceMatchMode {
  faceId,
  faceIdFaceString,
  faceFace,
}

extension FaceMatchModeExtension on FaceMatchMode {
  /// Returns the string representation of the face matching mode.
  ///
  /// The string representation is used to identify the face matching mode in API calls or configurations.
  String get faceMatchModeString {
    switch (this) {
      case FaceMatchMode.faceFace:
        return 'FACE_FACE';
      case FaceMatchMode.faceIdFaceString:
        return 'FACE_ID_FACE_STRING';
      default:
        return 'FACE_ID';
    }
  }
}

/// HVNetworkHelper is a utility class that provides methods for making OCR and face match API calls.
class HVNetworkHelper {
  static const MethodChannel _hvNetworkHelperChannel =
      MethodChannel(HyperSnapSDKConstants.hyperSnapChannel);

  /// Makes an OCR API call using the document image.
  ///
  /// Parameters:
  /// - `endpoint`: The endpoint URL to obtain the OCR response.
  /// - `documentUri`: The path/URI of the document image on which the OCR call has to be performed.
  /// - `parameters`: A map/json containing key-value pairs of different fraud/malice checks provided by Hyperverge.co.
  /// - `headers`: A map/json containing key-value pairs of different header options provided by Hyperverge.co.
  /// - `onComplete`: A callback function `Function(HVResponse?, HVError?)` that will be called when the OCR call is complete.
  static Future<void> makeOCRCall({
    required String endpoint,
    required String documentUri,
    required Map parameters,
    required Map headers,
    required Function(HVResponse?, HVError?) onComplete,
  }) async {
    try {
      final resMap = await _hvNetworkHelperChannel.invokeMethod(
        HyperSnapSDKConstants.networkHelperMakeOCRCall,
        {
          'endpoint': endpoint,
          'documentUri': documentUri,
          'parameters': parameters,
          'headers': headers,
        },
      );

      final errorObj = resMap['errorObj'];
      final resObj = resMap['resultObj'];
      final hvError = errorObj.isEmpty ? null : HVError.fromMap(errorObj);
      final hvResponse = resObj.isEmpty ? null : HVResponse.fromMap(resObj);

      onComplete(hvResponse, hvError);
    } catch (e) {
      log(e.toString());
    }
  }

  /// Makes a face match API call using a face image and a document image.
  ///
  /// Parameters:
  /// - `endpoint`: The endpoint URL to obtain the face match response.
  /// - `faceUri`: The path/URI of the face image on which the face match call has to be performed.
  /// - `documentUri`: The path/URI of the document image on which the face match call has to be performed.
  /// - `parameters`: A map/json containing key-value pairs of different fraud/malice checks provided by Hyperverge.co.
  /// - `headers`: A map/json containing key-value pairs of different header options provided by Hyperverge.co.
  /// - `faceMatchMode`: The mode to be used for face matching. Default value is `FaceMatchMode.faceId`.
  /// - `onComplete`: A callback function `Function(HVResponse?, HVError?)` that will be called when the face match call is complete.
  static Future<void> makeFaceMatchCall({
    required String endpoint,
    required String faceUri,
    required String documentUri,
    required Map parameters,
    required Map headers,
    FaceMatchMode faceMatchMode = FaceMatchMode.faceId,
    required Function(HVResponse?, HVError?) onComplete,
  }) async {
    try {
      final resMap = await _hvNetworkHelperChannel.invokeMethod(
        HyperSnapSDKConstants.networkHelperMakeFaceMatchCall,
        {
          'endpoint': endpoint,
          'faceUri': faceUri,
          'documentUri': documentUri,
          'faceMatchMode': faceMatchMode.faceMatchModeString,
          'parameters': parameters,
          'headers': headers,
        },
      );

      final errorObj = resMap['errorObj'];
      final resObj = resMap['resultObj'];
      final hvError = errorObj.isEmpty ? null : HVError.fromMap(errorObj);
      final hvResponse = resObj.isEmpty ? null : HVResponse.fromMap(resObj);

      onComplete(hvResponse, hvError);
    } catch (e) {
      log(e.toString());
    }
  }
}
