import 'dart:convert';

/// HVError represents an error object with an error code and error message.
class HVError {
  /// The error code associated with the error.
  final int errorCode;

  /// The error message describing the error.
  final String errorMessage;

  /// Creates an instance of [HVError].
  ///
  /// Parameters:
  /// - `errorCode`: The error code associated with the error.
  /// - `errorMessage`: The error message describing the error.
  HVError({
    required this.errorCode,
    required this.errorMessage,
  });

  /// Creates an instance of [HVError] from a map object.
  ///
  /// Parameters:
  /// - `errorObj`: A map containing the error details, where 'errorCode' and 'errorMessage' are required keys.
  ///
  /// Returns:
  /// - An instance of [HVError] with the error details from the map.
  static HVError fromMap(Map errorObj) {
    return HVError(
      errorCode: errorObj['errorCode'],
      errorMessage: errorObj['errorMessage'],
    );
  }
}

/// HVResponse represents a response object containing various attributes related to an API response.
class HVResponse {
  /// The URI of the image associated with the response.
  final String? imageUri;

  /// The URI of the video associated with the response.
  final String? videoUri;

  /// The URI of the full image associated with the response.
  final String? fullImageUri;

  /// The API result returned in the response.
  final Map? apiResult;

  /// The headers received in the API response.
  final Map? apiHeaders;

  /// The retake message associated with the response.
  final String? retakeMessage;

  /// The action associated with the response.
  final String? action;

  /// The active liveness result associated with the response.
  final Map? activeLivenessResult;

  /// The list of retake attempt responses.
  final List? retakeAttemptResponses;

  /// The raw barcode associated with the response.
  final String? rawBarcode;

  /// Creates an instance of [HVResponse].
  ///
  /// Parameters:
  /// - `imageUri`: The URI of the image associated with the response.
  /// - `fullImageUri`: The URI of the full image associated with the response.
  /// - `videoUri`: The URI of the video associated with the response.
  /// - `apiResult`: The API result returned in the response.
  /// - `apiHeaders`: The headers received in the API response.
  /// - `retakeMessage`: The retake message associated with the response.
  /// - `action`: The action associated with the response.
  /// - `activeLivenessResult`: The active liveness result associated with the response.
  /// - `retakeAttemptResponses`: The list of retake attempt responses.
  /// - `rawBarcode`: The barcode associated with the response.
  HVResponse({
    this.imageUri,
    this.fullImageUri,
    this.videoUri,
    this.apiResult,
    this.apiHeaders,
    this.retakeMessage,
    this.action,
    this.activeLivenessResult,
    this.retakeAttemptResponses,
    this.rawBarcode,
  });

  /// Converts the active liveness response URLs into a structured format.
  ///
  /// Parameters:
  /// - `activeLivenessUrls`: A map containing the active liveness URLs.
  ///
  /// Returns:
  /// - A structured map representing the active liveness response.
  static Map<String, dynamic> getStructuredActiveLivenessResponse({
    Map? activeLivenessUrls,
  }) {
    Map<String, dynamic> structuredActiveLivenessResponse = {};
    if (activeLivenessUrls == null) {
      return structuredActiveLivenessResponse;
    }
    if (activeLivenessUrls.isNotEmpty) {
      structuredActiveLivenessResponse['live'] = 'yes';

      List<Map<String, String>> gestures = [];

      // Traverse the straight gesture URLs
      if (activeLivenessUrls.containsKey('STRAIGHT_GESTURE')) {
        for (String url in activeLivenessUrls['STRAIGHT_GESTURE']) {
          gestures.add({'prompt': 'straight', 'path': url});
        }
      }

      // Traverse the right gesture URLs
      if (activeLivenessUrls.containsKey('RIGHT_GESTURE')) {
        for (String url in activeLivenessUrls['RIGHT_GESTURE']) {
          gestures.add({'prompt': 'right', 'path': url});
        }
      }

      // Traverse the left gesture URLs
      if (activeLivenessUrls.containsKey('LEFT_GESTURE')) {
        for (String url in activeLivenessUrls['LEFT_GESTURE']) {
          gestures.add({'prompt': 'left', 'path': url});
        }
      }

      structuredActiveLivenessResponse['gestures'] = gestures;
    }

    return structuredActiveLivenessResponse;
  }

  /// Creates an instance of [HVResponse] from a map object.
  ///
  /// Parameters:
  /// - `resObj`: A map containing the response details.
  ///
  /// Returns:
  /// - An instance of [HVResponse] with the response details from the map.
  static HVResponse fromMap(Map responseMap) {
    var apiResultObj = (responseMap['apiResult'] != null)
        ? jsonDecode(responseMap['apiResult'])
        : {};
    var apiHeadersObj = (responseMap['apiHeaders'] != null)
        ? Map<String, dynamic>.from(jsonDecode(responseMap['apiHeaders']))
        : {};

    return HVResponse(
      imageUri: responseMap['imageUri'],
      fullImageUri: responseMap['fullImageUri'],
      videoUri: responseMap['videoUri'],
      retakeMessage: responseMap['retakeMessage'],
      action: responseMap['action'],
      retakeAttemptResponses: responseMap['retakeAttemptResponses'],
      activeLivenessResult: getStructuredActiveLivenessResponse(
        activeLivenessUrls: responseMap['gestureLivenessUrls'],
      ),
      rawBarcode: responseMap['rawBarcode'],
      apiResult: apiResultObj,
      apiHeaders: apiHeadersObj,
    );
  }
}
