import 'dart:developer';

import 'package:flutter/services.dart';
import 'package:hypersnapsdk_flutter/hyper_snap_sdk_constants.dart';

import 'hv_ui_config.dart';

/// Enum representing different regions.
enum Region {
  /// India region.
  india,

  /// Asia Pacific region.
  asiaPacific,

  /// United States region.
  unitedStates,

  /// Africa region.
  africa,
}

/// Extension on [Region] enum to provide additional functionality.
extension RegionExtension on Region {
  /// Returns the name of the region.
  ///
  /// Returns:
  /// - A string representing the name of the region.
  String get regionName {
    switch (this) {
      case Region.india:
        return 'INDIA';
      case Region.africa:
        return 'AFRICA';
      case Region.unitedStates:
        return 'UNITED_STATES';
      case Region.asiaPacific:
        return 'ASIA_PACIFIC';
      default:
        return 'INDIA';
    }
  }
}

/// HyperSnapSDK is used to initial the sdk, to config the sdk, and to manage user sessions
class HyperSnapSDK {
  static const MethodChannel _hyperSnapSDKChannel =
      MethodChannel(HyperSnapSDKConstants.hyperSnapChannel);

  /// Initializes the HyperSnap SDK inside the client's application.
  ///
  /// Parameters:
  /// - [appId]: The application ID provided by HyperVerge.co.
  /// - [appKey]: The application key provided by HyperVerge.co.
  /// - [region]: The region in which the SDK should be initialized. (e.g., [Region.india])
  ///
  /// Returns a [Future] that completes with a [String] value of "initialized" if the initialization is successful.
  static Future<String> initialize(
      String appId, String appKey, Region region) async {
    String result = "Not Initialized";
    try {
      result = await _hyperSnapSDKChannel
          .invokeMethod(HyperSnapSDKConstants.initialize, {
        HyperSnapSDKConstants.appId: appId,
        HyperSnapSDKConstants.appKey: appKey,
        HyperSnapSDKConstants.region: region.regionName,
      });
    } catch (e) {
      log(e.toString());
    }
    return result;
  }

  /// Initializes the HyperSnap SDK with an access token inside the client's application.
  ///
  /// Parameters:
  /// - [appId]: The application ID provided by HyperVerge.co.
  /// - [accessToken]: The access token provided by HyperVerge.co.
  /// - [region]: The region in which the SDK should be initialized. (e.g., Region.india)
  ///
  /// Returns a [Future] that completes with a [String] value of "initialized" if the initialization is successful.
  static Future<String> initializeWithToken(
      String appId, String accessToken, Region region) async {
    String result = "Not Initialized";

    try {
      result = await _hyperSnapSDKChannel
          .invokeMethod(HyperSnapSDKConstants.initializeWithToken, {
        HyperSnapSDKConstants.appId: appId,
        HyperSnapSDKConstants.accessToken: accessToken,
        HyperSnapSDKConstants.region: region.regionName,
      });
    } catch (e) {
      log(e.toString());
    }
    return result;
  }

  /// Sets the configuration options for the HyperSnap SDK.
  ///
  /// Parameters:
  /// - [hyperSnapSDKConfig]: The configuration options to be set.
  static setHyperSnapSDKConfig(
      {required HyperSnapSDKConfig hyperSnapSDKConfig}) async {
    try {
      await _hyperSnapSDKChannel.invokeMethod(
        HyperSnapSDKConstants.hyperSnapSDKConfig,
        hyperSnapSDKConfig.toMap(),
      );
    } catch (e) {
      log(e.toString());
    }
  }

  /// Starts the user session in the application.
  ///
  /// Parameters:
  /// - [userID]: The user ID associated with the session.
  ///
  /// Returns a [Future] that completes with a [bool] value of `true` if the user session has been started successfully.
  static Future<bool> startUserSession(String userID) async {
    bool flag = false;
    try {
      flag = await _hyperSnapSDKChannel
          .invokeMethod(HyperSnapSDKConstants.startUserSession, {
        HyperSnapSDKConstants.userId: userID,
      });
    } catch (e) {
      log(e.toString());
    }
    return flag;
  }

  /// Ends the user session if any session is active.
  static endUserSession() async {
    try {
      await _hyperSnapSDKChannel
          .invokeMethod(HyperSnapSDKConstants.endUserSession);
    } catch (e) {
      log(e.toString());
    }
  }

  /// Sorts the keys of the response object alphabetically.
  ///
  /// Parameters:
  /// - [map]: The response object to be sorted.
  ///
  /// Returns a [Future] that completes with a [String] value of the sorted object.
  static Future<String> sortJSONKeysAlphabetically(Map map) async {
    String string = "";
    try {
      string = await _hyperSnapSDKChannel
          .invokeMethod(HyperSnapSDKConstants.sortJSONKeysAlphabetically, {
        "map": map,
      });
    } catch (e) {
      log(e.toString());
    }
    return string;
  }
}

/// Configuration class for setting options for the HyperSnap SDK.
class HyperSnapSDKConfig {
  /// Determines whether SSL pinning should be enabled.
  bool? shouldEnableSSLPinning;

  /// Determines whether signature should be used for API calls involving images.
  bool? shouldUseSignature;

  /// Determines whether raw response should be returned from the plugin.
  bool? shouldReturnRawResponse;

  /// Timeout configuration for API calls.
  HVTimeoutConfig? timeoutConfig;

  /// UI configuration for the HyperSnap SDK.
  HVUIConfig? hvUIConfig;

  /// Creates a new instance of [HyperSnapSDKConfig] with the specified options.
  ///
  /// Parameters:
  /// - [shouldEnableSSLPinning]: (optional) Set to true if SSL pinning should be enabled in the application.
  /// - [shouldUseSignature]: (optional) Set to true if the signature of the images should be used for API calls.
  /// - [shouldReturnRawResponse]: (optional) Set to true if raw response should be returned from the plugin.
  /// - [timeoutConfig]: (optional) Timeout configuration for API calls.
  /// - [hvUIConfig]: (optional) UI configuration for the HyperSnap SDK.
  HyperSnapSDKConfig({
    this.shouldEnableSSLPinning,
    this.shouldReturnRawResponse,
    this.shouldUseSignature,
    this.timeoutConfig,
    this.hvUIConfig,
  });

  /// Converts the [HyperSnapSDKConfig] object to a Map.
  Map<String, dynamic> toMap() {
    return {
      HyperSnapSDKConstants.shouldEnableSSLPinning: shouldEnableSSLPinning,
      HyperSnapSDKConstants.shouldUseSignature: shouldUseSignature,
      HyperSnapSDKConstants.shouldReturnRawResponse: shouldReturnRawResponse,
      HyperSnapSDKConstants.connectTimeout: timeoutConfig?.connectTimeout,
      HyperSnapSDKConstants.readTimeout: timeoutConfig?.readTimeout,
      HyperSnapSDKConstants.writeTimeout: timeoutConfig?.writeTimeout,
      HyperSnapSDKConstants.uiConfigMap: hvUIConfig?.toJSONString(),
    };
  }
}

/// Configuration class for setting timeout values for API calls.
class HVTimeoutConfig {
  /// Connection timeout value in milliseconds.
  final int connectTimeout;

  /// Read timeout value in milliseconds.
  final int readTimeout;

  /// Write timeout value in milliseconds.
  final int writeTimeout;

  /// Creates a new instance of [HVTimeoutConfig] with the specified timeout values.
  ///
  /// Parameters:
  /// - [connectTimeout]: Connection timeout value in milliseconds.
  /// - [readTimeout]: Read timeout value in milliseconds.
  /// - [writeTimeout]: Write timeout value in milliseconds.
  HVTimeoutConfig({
    required this.connectTimeout,
    required this.readTimeout,
    required this.writeTimeout,
  });
}
